const path = require('path');

module.exports = {
  env: {
    browser: true,
    commonjs: true,
    es6: true,
    jest: true,
    node: true,
  },
  globals: {
    document: false,
    navigator: false,
    window: false,
  },
  parser: 'babel-eslint',

  parserOptions: {
    ecmaVersion: 8,
    sourceType: 'module',
    ecmaFeatures: {
      experimentalObjectRestSpread: true,
      generators: true,
      jsx: true,
    },
  },

  extends: [
    'eslint:recommended',
    'plugin:flowtype/recommended',
    'plugin:jest/recommended',
    'plugin:jsx-a11y/recommended',
    'plugin:promise/recommended',
    'plugin:react/recommended',
    'plugin:security/recommended',
    'airbnb',
  ],

  settings: {
    'import/ignore': ['node_modules'],
    'import/extensions': ['.js', '.json'],
    'import/resolver': {
      node: {
        paths: [path.resolve(__dirname, './src')],
      },
    },
  },

  plugins: [
    'flowtype',
    'jest',
    'jsx-a11y',
    'import',
    'node',
    'promise',
    'react',
    'security',
    'standard',
  ],

  rules: {
    'arrow-spacing': ['warn', { before: true, after: true }],
    'constructor-super': 'warn',
    'generator-star-spacing': ['warn', { before: true, after: true }],
    'handle-callback-err': ['warn', '^(err|error)$'],
    indent: ['warn', 2],
    'jsx-quotes': ['warn', 'prefer-single'],
    'keyword-spacing': ['warn', { before: true, after: true }],
    'new-cap': ['warn', { newIsCap: true, capIsNew: false }],
    'new-parens': 'warn',
    'no-array-constructor': 'warn',
    'no-class-assign': 'warn',
    'no-cond-assign': ['warn', 'always'],
    'no-confusing-arrow': 'warn',
    'no-console': 'off',
    'no-compare-neg-zero': 'warn',
    'no-const-assign': 'warn',
    'no-dupe-class-members': 'warn',
    'no-extra-parens': ['off', 'all', {
      conditionalAssign: true,
      nestedBinaryExpressions: false,
      returnAssign: false,
    }],
    'no-iterator': 'warn',
    'no-mixed-operators': ['warn', {
      allowSamePrecedence: true,
      groups: [
        ['&', '|', '^', '~', '<<', '>>', '>>>'],
        ['==', '!=', '===', '!==', '>', '>=', '<', '<='],
        ['&&', '||'],
        ['in', 'instanceof']],
    }],
    'no-multiple-empty-lines': ['warn', { max: 1, maxEOF: 0 }],
    'no-new-object': 'warn',
    'no-new-symbol': 'warn',
    'no-obj-calls': 'warn',
    'no-regex-spaces': 'warn',
    'no-restricted-syntax': ['warn', 'LabeledStatement', 'WithStatement'],
    'no-tabs': 'warn',
    'no-template-curly-in-string': 'warn',
    'no-this-before-super': 'warn',
    'no-trailing-spaces': 'warn',
    'no-undef': 'warn',
    'no-unexpected-multiline': 'warn',
    'no-unneeded-ternary': ['warn', { defaultAssignment: false }],
    'no-unreachable': 'warn',
    'no-unused-expressions': ['warn', { allowShortCircuit: true, allowTernary: true, allowTaggedTemplates: true }],
    'no-useless-computed-key': 'warn',
    'no-useless-constructor': 'warn',
    'no-useless-rename': ['warn', { ignoreDestructuring: false, ignoreImport: false, ignoreExport: false }],
    'no-useless-return': 'warn',
    'no-whitespace-before-property': 'warn',
    'object-property-newline': ['warn', { allowMultiplePropertiesPerLine: true }],
    'one-var': ['warn', { initialized: 'never' }],
    'operator-assignment': ['warn', 'always'],
    'operator-linebreak': ['warn', 'after', { overrides: { '?': 'before', ':': 'before' } }],
    'padded-blocks': ['warn', { blocks: 'never', switches: 'never', classes: 'never' }],
    'prefer-const': 'warn',
    'prefer-promise-reject-errors': 'warn',
    'prefer-template': 'warn',
    quotes: ['warn', 'single', { avoidEscape: true, allowTemplateLiterals: true }],
    'require-yield': 'warn',
    'rest-spread-spacing': ['warn', 'never'],
    semi: ['warn', 'always'],
    'semi-spacing': ['warn', { before: false, after: true }],
    'semi-style': ['warn', 'last'],
    'space-before-function-paren': ['warn', 'always'],
    'spaced-comment': ['warn', 'always', {
      line: {
        markers: ['*package', '!', '/', ','],
      },
      block: {
        balanced: true,
        markers: ['*package', '!', ',', ':', '::', 'flow-include'],
        exceptions: ['*'],
      },
    }],
    'symbol-description': 'warn',
    'template-curly-spacing': ['warn', 'never'],
    'template-tag-spacing': ['warn', 'never'],
    'wrap-iife': ['warn', 'any', { functionPrototypeMethods: true }],
    'yield-star-spacing': ['warn', 'both'],
    yoda: ['warn', 'never'],

    'import/default': 'warn',
    'import/export': 'warn',
    'import/first': 'warn',
    'import/named': 'warn',
    'import/namespace': 'warn',
    'import/no-amd': 'warn',
    'import/no-duplicates': 'warn',
    'import/no-extraneous-dependencies': 'warn',
    'import/no-named-as-default': 'warn',
    'import/no-named-as-default-member': 'warn',
    'import/no-unresolved': ['warn', { commonjs: true }],
    'import/no-webpack-loader-syntax': 'warn',

    'jsx-a11y/aria-props': 'warn',
    'jsx-a11y/heading-has-content': 'warn',
    'jsx-a11y/label-has-for': 'warn',
    'jsx-a11y/mouse-events-have-key-events': 'warn',
    'jsx-a11y/aria-role': 'warn',
    'jsx-a11y/role-has-required-aria-props': 'warn',
    'jsx-a11y/role-supports-aria-props': 'warn',
    'jsx-a11y/img-redundant-alt': 'warn',
    'jsx-a11y/no-access-key': 'warn',
    'jsx-a11y/href-no-hash': ['off', { components: ['a'] }], // fix 'definition not found warnings'

    'node/no-deprecated-api': 'warn',
    'node/process-exit-as-throw': 'warn',

    'promise/param-names': 'warn',

    'react/forbid-prop-types': 'off',
    'react/jsx-boolean-value': 'warn',
    'react/jsx-curly-spacing': ['warn', 'never'],
    'react/jsx-equals-spacing': ['warn', 'never'],
    'react/jsx-first-prop-new-line': 'off',
    'react/jsx-indent': ['warn', 2],
    'react/jsx-indent-props': ['warn', 2],
    'react/jsx-no-bind': ['warn', { allowArrowFunctions: true, allowBind: false, ignoreRefs: true }],
    'react/jsx-no-duplicate-props': ['warn', { ignoreCase: true }],
    'react/jsx-no-undef': 'warn',
    'react/jsx-pascal-case': ['warn', { allowAllCaps: true, ignore: [] }],
    'react/jsx-uses-vars': 'warn',
    'react/jsx-uses-react': 'warn',
    'react/no-danger-with-children': 'warn',
    'react/no-deprecated': 'warn',
    'react/no-direct-mutation-state': 'warn',
    'react/no-did-update-set-state': 'warn',
    'react/no-is-mounted': 'warn',
    'react/no-unknown-property': 'warn',
    'react/no-unused-prop-types': 'warn',
    'react/prop-types': 'warn',
    'react/react-in-jsx-scope': 'warn',
    'react/require-render-return': 'warn',
    'react/self-closing-comp': 'warn',
    'react/style-prop-object': 'warn',

    'standard/array-bracket-even-spacing': ['warn', 'either'],
    'standard/computed-property-even-spacing': ['warn', 'even'],
    'standard/no-callback-literal': 'warn',
    'standard/object-curly-even-spacing': ['warn', 'either'],

    'flowtype/define-flow-type': 'warn',
    'flowtype/require-valid-file-annotation': 'warn',
    'flowtype/use-flow-type': 'warn',
  },
};
